#include "stm32f10x.h"
#include "grafiki.h"



//-----------------------------------------------------------------------------------------------------------------------------
//-------------- Definicje serwisowe ------------------------------------------------------------------------------------------
#define enablewatchdog      0     //Zalaczenie lub wylaczenie ukladu watchdog
#define testtouchpad        0    //Zalaczenie lub wylaczenie funkcji testujacej panel dotykowy
#define showlogo            1    //zezwolenie na narysowanie loga na wyswietlaczu na ekranie glownym
//-------------- Definicje uzytkowe ------------------------------------------------------------------------------------------
#define  activcalib        (GPIOA->IDR & GPIO_Pin_0)  //definicja aktywowania kalibracji
#define  clearwdt          IWDG_ReloadCounter()
#define  regonoff          (GPIOC->IDR & GPIO_Pin_6)  //definicja aktywowania kalibracji

//---------------------------------------------------------------------------------------------------------------------------
//Definicje kalibracyjne
#define mfvoltage 3000      //aby zwiekszyc wyswietlana aktualna wartosc to nalezy zwiekszyc mfvoltage lub mfcurrent
#define mfcurrent 2000
#define dacfvoltage 3000    //aby zwiekszyc obliczana wartosc DAC nalezy zmniejszyc wartosc dacfvoltage tak samo aby zwiekszyc
#define dacfcurrent 2000    //dac dla wartosci pradu to nalezy zmniejszyc dacfcurrent
#define deltaregulation 10  //okreslenie wartosci dopuszczalnej odchylki dla regulacji, jezeli roznica jest wieksza wtedy nastapi regulacja
//-------------------- Definicje obslugi multipleksera analogowego -----------------------------------------------------------
#define  onAmux              GPIOC -> BSRR = GPIO_Pin_0
#define  offAmux             GPIOC -> BRR = GPIO_Pin_0
#define  onBmux              GPIOC -> BSRR = GPIO_Pin_1
#define  offBmux             GPIOC -> BRR = GPIO_Pin_1
#define  onCmux              GPIOC -> BSRR = GPIO_Pin_2
#define  offCmux             GPIOC -> BRR = GPIO_Pin_2
//---------------------Definicje dla INFOLED oraz BUZZER ---------------------------------------------------------------------
#define  onLED              GPIOC -> BSRR = GPIO_Pin_8
#define  offLED				GPIOC -> BRR = GPIO_Pin_8
#define  onBUZZER			GPIOC -> BSRR = GPIO_Pin_9
#define  offBUZZER			GPIOC -> BRR = GPIO_Pin_9
//---------------------Definicje sterowania przetwornica ---------------------------------------------------------------------
#define onconverter         GPIOB -> BSRR = GPIO_Pin_10
#define offconverter        GPIOB -> BRR =  GPIO_Pin_10

//----------------------------------------------------------------------------------------------------------------------------
//------------------ Deklaracje funkcji systemowych  -------------------------------------------------------------------------
void GPIO_Config(void);
void RCC_Config(void);
void NVIC_Config(void);
void SPI_init(void);
void Flash_odczyt2(void);
void Flash_zapis2(void);
void TIM_Config(void);
void IWDT_config(void);
void DMA_Config(void);
void ADC_Config(void);
void DAC_Config(void);
//----------------------------------------------------------------------
//-------- Deklaracje funkcji uzytkowych -------------------------------
uint8_t gettouchvalue(void);
void calibrationtouch(void);
void dimingled(uint8_t diming);
void testingtouch(void);
void controlmux(uint8_t canal);
void putvaluecurrentvoltage(uint8_t *firstdraw, uint16_t *current, uint16_t *voltage);
void valueV(uint16_t x, uint16_t y, uint16_t *measure, uint8_t przecinek);
void valueA(uint16_t x, uint16_t y, uint16_t *measure, uint8_t przecinek);
void animationtouch(uint16_t x, uint16_t y, uint8_t touchclose);
void touchmenu0(void);
void touchmenu1(void);
void putincdec(uint16_t x, uint16_t y, uint8_t *show, uint8_t* firsdraw2);
void putactualvalue(uint16_t x, uint16_t y, uint8_t *firstdraw3, uint8_t *zaznaczenie, uint8_t *show, uint8_t *changevalue);
void checkvalue(void);
void tempsetvalue(void);
void multimeter(uint16_t *valcur, uint16_t *valvolt);
void setdacs(uint16_t currentdac, uint16_t voltagedac);
void controlLB(uint16_t *current, uint8_t *reg);
void controlregulation(uint8_t *regulation);
void changerange(uint8_t *changerange);
void doregulation(uint8_t *regulation, uint8_t *changerange, uint16_t setvalue, uint16_t actualvalue, uint8_t *firstuse);
void setbacklight(uint8_t *firstdraw, uint8_t percent);
//----------------------------------------------------------------------
//------- Funkcje obsugi pamieci flash --------------------------------
void Flash_odczyt2(void);
void Flash_zapis2(void);
void Flash_zapisustawien(void);
void Flash_odczytustawien(void);
//-------------------------------------------------------------------------
//Zmienne programowe
volatile uint32_t TIMERUS, touchms, Xtouch, Ytouch, timeranime, timeranime2, timerelmdrgtouch, animetimer1, animetimer2, timervalue, timerbuzz, timerled, timerrange,
                  timerregulation;
const uint16_t xcal1=4000, xcal2=100, ycal1=4000, ycal2=100;
float aX, bX, dX, aY, bY, dY;   //zmienne potrzebne przy pelnej kalibracji
uint16_t buftx[4];
uint16_t bufty[4];
uint8_t wykalibrowano, globalk=0, globalk2=0, flagaponownejkalibracji, menu, varfirstdraw1=1, varshow, varfirstdraw2=1, varupdown, varselect, varfirstdraw3,
		varchangevalue, varregulation, varrange, varfirstregulation, globalflag, backlight=0, varbacklight;
int chosefont, rotate;
uint16_t valueADC[2];   //zmienna potrzebna w ukladzie DMA do ktorej jest przepisywana wartosc z przetwornika
uint32_t *wskkalib, *wskustawien, tempsetvoltage, tempsetcurrent;
uint16_t valueCURRENT=2000, valueVOLTAGE=3000, ttvoltage, ttcurrent, backtemp;
uint32_t numbers[8];

const uint16_t changecolors[9]={0x0B34, 0x0AD2, 0x0A90, 0x0A4E, 0x0A2D, 0x01CB, 0x0189,  0x00C5, 0x0000};
const uint16_t ranges[9]={0, 500, 700, 1000, 1400, 1700, 2100, 2600, 3200};   //zakresy napiec dla regulatora impulsowego


//Zapisane aktualnego ustawienia oraz innych parametrow w pamieci flash
uint32_t flashzapis2[3]={1500, 1000, 100};


//-----------------------------------------------------------------------------------------------------------------------------
//---------------------- Definicje dla pamieci flash mikrokontroler -----------------------------------------------------------
//-----------------------------------------------------------------------------------------------------------------------------
uint32_t EraseCounter = 0x00, Address = 0x00;
uint32_t NbrOfPage = 0x00;
volatile FLASH_Status FLASHStatus=FLASH_COMPLETE;

#define BaseAdresFlash 0x08000000

//ofsety osobno dla funkcji - dla zapamietywania wartosci kalibracyjnych touchapnelu
#define OffsetNazwy2 0x39C00
#define BANK1_WRITE_START_ADDR2  ((uint32_t)0x08039C00)  //Ustawienie 1kb
#define BANK1_WRITE_END_ADDR2    ((uint32_t)0x0803A000)
#define FLASH_PAGE_SIZE         ((uint16_t)0x400)

//ofsety dla zapisywania zmiennych ustawien uzytkownika
#define OffsetNazwy 0x3A400
#define BANK1_WRITE_START_ADDR  ((uint32_t)0x0803A400)  //Ustawienie 1kb
#define BANK1_WRITE_END_ADDR    ((uint32_t)0x0803A800)
//-------------------------------------------------------------------------



int main(void)
{
//-------------------------------------------------------------
//Inicjalizacja procesora oraz peryferiow i urzadzen zewnetrznych
	  RCC_Config();
	  NVIC_Config();
	  GPIO_Config();
	  SPI_init();
	  TIM_Config();

	  //zalaczenie przetwornika ADC oraz ukladu DMA odpowiedzialnych za pomiar
	  ADC_Config();
	  DMA_Config();
	  DAC_Config();


#if enablewatchdog==1
	  IWDT_config();
	  clearwdt;
#endif


//---------------------------------------
//Inicjalizacja zegrar systemowego systick
	if (SysTick_Config_Mod(SysTick_CLKSource_HCLK_Div8, 900)) //przerwanie co 0,1ms
	{
		while(1);		// W razie bledu petla nieskonczona
	}

	 ILI9341_InitLCD();  //Inicjalizacja TFT
	 XPT2046_SPI_Init(); //inicjalizacja panelu dotykowego
	 ILI9341_Clear(White); //Wyczyszczenie wywietlacza biaym tem


//--------------------------------------------------------------
//Fragment programu odpowiedzialny za sprawdzenie czy
//bedzie przeprowadzana kalibracja

	 wskkalib=(uint32_t *)(BaseAdresFlash + OffsetNazwy2);  //ustawienie wskaznika na adres komorki ktora jest flaga - czy juz byla doknana kalibracja po pierwszym uruchomienu

	 if(*wskkalib!=0x256faba4)
	    {
	    	 flagaponownejkalibracji=1;
	  	}

	 if(activcalib)  //jezeli jest wcisniet przycisk odpowiedzialny za ponowna kalibracje touch panelu to przed przy uruchomieniu do kalibracji
	    {
	    	 flagaponownejkalibracji=1;
	  	}

	 if(flagaponownejkalibracji)
	    {
	    	  TIM3->CCR3=7999;   //jezeli kalibrujemy touchpanel to zaswiecenie wyswietlacza z maksymalna jasnoscia
	          calibrationtouch();
	    	  flagaponownejkalibracji=0;
	   	}

//-----------------------------------------------------------
//Fragment odpowiedzialny za sprawdzenie czy juz ustawienia byly przeniesione do pamieci flash jezeli nie to nalezy przeniesc ustawienia do pamieci flash

	    wskustawien=(uint32_t *)(BaseAdresFlash + OffsetNazwy);  //ustawienie wskaznika na adres komorki ktora jest flaga czy juz byl dokonany zapis i pierwsze uruchonmienie

	    if(*wskustawien!=0x55aef09b)
	     {
	    	Flash_zapisustawien();    //zapisanie do flash wszystkich ustawien uzytkownika
	     }

	    //w tym miejscu nastepuje odczyt wspolczynnikow kalibracji z pamieci flash
	     Flash_odczyt2();
	  	//odczyt wszystkich ustawien z pamieci flash
	     Flash_odczytustawien();

	     wykalibrowano=1;  //ustawienie flagi zezwalajacej prace touchpana ze wspolczynnikami korygujacymmi

//Funkcja ustawiajaca jasnosc wywietlacza
//argument funkcji jest zalezny od wybranej opcji
	     dimingled(flashzapis2[2]);


//Po uruchomieniu wywietlenie logo firmowego
#if showlogo==1
	     ILI9341_Clear(White); //Wyczyszczenie wywietlacza biaym tem
	     putpictures2(80, 40, 160, 160, Logo);

	     TIMERUS=30000;
	     while(TIMERUS)
	     {
#if enablewatchdog==1
	    	 clearwdt;
#endif

	    	 if(gettouchvalue())  //sprawdzenie czy nacisnieto panel dotykowy
	    	    	 	    	  {
	    	    	 	    		 if(Xtouch>100 && Xtouch<220 && Ytouch>80 && Ytouch<160)
	    	    	 	    		 			{
	    	    	 	    			 	 	 	 backlight=1;
	    	    	 	    			 	 	 	 varbacklight=1;
	    	    	 	    		 		    }
	    	    	 	    	  }
	     }



#endif

//----------------------------------------------------------------
//----------------- Zmiana podswietlenia wyswietlacza ------------
	     while(backlight)
	     {

#if enablewatchdog==1
	    	 clearwdt;
#endif

	    	setbacklight(&varbacklight, flashzapis2[2]);

	    	 if(gettouchvalue())  //sprawdzenie czy nacisnieto panel dotykowy
	    	    	  {
	    	    		 if(Xtouch>120 && Xtouch<200 && Ytouch>170 && Ytouch<220)
	    	    		 			{
	    	    			 	 	 	 backlight=0;
	    	    			 	 	 	 Flash_zapisustawien();    //zapisanie do flash wszystkich ustawien uzytkownika
	    	    		 		    }
	    	    		 if(Xtouch>0 && Xtouch<320 && Ytouch>80 && Ytouch<160)
	    	    		 	    		 			{
	    	    			                           if(Xtouch>=310) backtemp=300;
	    	    			                           else if(Xtouch<10) backtemp=0;
	    	    			                           else backtemp=Xtouch-10;

	    	    			                           backtemp=backtemp/3;
	    	    			                           flashzapis2[2]=backtemp;

	    	    			                           dimingled(flashzapis2[2]);
	    	    		 	    		 			}
	    	    	  }
	     }
//----------------------------------------------------------------

	 ILI9341_Clear(White); //Wyczyszczenie wywietlacza biaym tem
//---------------------------------------------------------------
//--------- Funkcja testowa touchpanelu -------------------------
#if testtouchpad==1
testingtouch();
#endif
//------------------------------------------------------------------

//Uruchomienie konwersji dla przetwornika ADC
ADC_SoftwareStartConvCmd(ADC1, ENABLE);

//Przed wejsciem do programu ustawienie parametrow DAC
setdacs(flashzapis2[1], flashzapis2[0]);

varrange=1; //zezwoleni na ustawienie zakresu
varfirstregulation=1; //zezwolenie na ustawienie zmiennych w funkcji pierwsza regulacja
//------------------------------------------------------------------
//Gwna ptla programu
	while(1)
		{

		//-------------------------------------------------------------------------------------------------------------------
		//Menu glowne w ktorym nastepuje wyswietlanie wartosci pradu i napiecia oraz jest mozliwosc przejscia
		//do podmenu ustawien ograniczenia pradowego i napiecia
		while(!menu)
		{

#if enablewatchdog==1
	    	 clearwdt;
#endif
			 multimeter(&valueCURRENT, &valueVOLTAGE);  //Pobranie zmierzonych wartosci obliczenie sredniej z 10 oraz przeslanie do zmiennych

			 if(!timervalue)
			 {
			 putvaluecurrentvoltage(&varfirstdraw1, &valueCURRENT, &valueVOLTAGE);  //Funkcja piszaca aktualne wartosci pradu oraz napiecia
			 timervalue=1000; //odsiwezanie wartosci co 100ms
			 }
			 changerange(&varrange);

			 animationtouch(280, 199, 1);   //funkcja aniomwana dla ikony touch

			 controlLB(&valueCURRENT, &varregulation);

			 //Funkcje obslugujace doregulowywanie zasilacza - zalacza lub wylacza regulacje
			 controlregulation(&varregulation);
			 //Funkcja dokonujaca regulacji
			 doregulation(&varregulation, &varrange, flashzapis2[0], valueVOLTAGE, &varfirstregulation);

		     if(gettouchvalue())  //sprawdzenie czy nacisnieto panel dotykowy
			 	 {
		    	      touchmenu0();
			 	 }
		}
		//Podmenu w ktorym dokonujemy zmiany ustawien
		while(menu==1)
		{

#if enablewatchdog==1
	    	 clearwdt;
#endif

			 animationtouch(280, 199, 0);   //funkcja aniomwana dla ikony close

			 putincdec(275, 80, &varshow, &varfirstdraw2);
			 putactualvalue(57, 75, &varfirstdraw3, &varselect, &varshow, &varchangevalue);

			 if(gettouchvalue())  //sprawdzenie czy nacisnieto panel dotykowy
			 {
					    touchmenu1();
			 }
		}
		//----------------------------------------------------------------------------------------------------------------------

         } //Koniec petli nieskonczonosci
	//--------------------------------------------------------------------------------------

 } //Koniec funkcji main
//------------------------------------------------------------------


//------------------------------------------------------------------------------------------------------------------------------
//----------------------------------------- FUNKCJE SYSTEMOWE ------------------------------------------------------------------

void GPIO_Config(void)
{

  //konfigurowanie portow GPIO

  //Konfigurowanie portow spi dla wyswietlacza - SPI1
  GPIO_InitTypeDef  GPIO_InitStructure;

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_15;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIOA->BSRR = GPIO_Pin_15;

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOB, &GPIO_InitStructure);

  GPIOB->BSRR = GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5;

  //*********************************************************************************
  //Dla kontroli sterowania wyswietlacz Pin10 reset , Pin10 CSRW Pin11 ewentualny nSS
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11 | GPIO_Pin_10 | GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  GPIOA ->BSRR = GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11;


    //ustawienie portu odpowiedzialnego za kalibracje touchpanelu
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPD;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIOA ->BRR = GPIO_Pin_0;

    //Wyjscie dla ukladu PWM odpowiedzialnego za jasnosc wyswietlacza
   	 GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
   	 GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
   	 GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP; //wyjscie alternatywne o wysokiej impedancji
   	 GPIO_Init(GPIOB, &GPIO_InitStructure);

     //Wyjscie dla sterowania przetwornica impulsowa
     GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
     GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
     GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP; //wyjscie alternatywne o wysokiej impedancji
     GPIO_Init(GPIOB, &GPIO_InitStructure);

  	//************************************************************************
  	//Konfiguracja wejscia przetwornika jako wejscia analogowego - ADC
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_2;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
  	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  	GPIO_Init(GPIOA, &GPIO_InitStructure);



     //***************************************************************************
     //Inicjalizacja portow odpowiedzialnych za obsluge multipleksera analogowego
     GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2;
     GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
     GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
     GPIO_Init(GPIOC, &GPIO_InitStructure);

     //Zalaczenie portow i ustawienie multipleksera na kanlal X0
     onAmux;
     onBmux;
     onCmux;

	 GPIO_PinRemapConfig(GPIO_Remap_SWJ_JTAGDisable, ENABLE);   //Wylaczenie JTAG
	 GPIO_PinRemapConfig(GPIO_Remap_SPI1, ENABLE);
	 GPIO_PinLockConfig( GPIOB,GPIO_Pin_3 |  GPIO_Pin_4 | GPIO_Pin_5);

	  //****************************************************************************
	 //Piny sterujace LED oraz BUZZER
	    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9;
	    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	    GPIO_Init(GPIOC, &GPIO_InitStructure);

	 //-----------------------------------------------------------------------------
	//Konfiguracja wyjsc dla przetwornika DAC
	    GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_5 | GPIO_Pin_4;
	    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
	    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	    GPIO_Init(GPIOA, &GPIO_InitStructure);

	 //Konfiguracja portu odpowiedzialnego za zalaczenie lub wylaczenie regulacji DP-02 - 2
	        GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
	        GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPD;
	        GPIO_Init(GPIOC, &GPIO_InitStructure);


}


void NVIC_Config(void)
{

	NVIC_InitTypeDef    NVIC_InitStructure;

	NVIC_InitStructure.NVIC_IRQChannel =SysTick_IRQn ; //tutaj trzeba zwrocic uwage na definicje w pliu stm32f10x.h
	//******************************************************************************************************************
  	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  	NVIC_Init(&NVIC_InitStructure);
}


void SPI_init(void)
{

   SPI_InitTypeDef  SPI_InitStructure;    //definicja obiektu stuktury

   SPI_I2S_DeInit(SPI1);

   SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;    //ustawienie trybu 2 lini - MOSI i MISO
   SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;                     //ustawienie dlugosci danych przesylanych
   SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;                            //zegar w stanie niskim
   SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;                          //reakcja na pierwsze zbocze zegarowe
   SPI_InitStructure.SPI_NSS = SPI_NSS_Hard;							 //ustawienie obslugi wyjscia CS programowo
   SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_2;    //preskaler dla zegara taktujacego SPI - 48Mhz/2=24Mhz
   SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;   				 //ustawienie najbardziej znacxacego bitu jako wysylany pierwszy
   SPI_InitStructure.SPI_CRCPolynomial = 0;                              //kontrola przeslanych bitow
   SPI_InitStructure.SPI_Mode = SPI_Mode_Master;   						 //tryb master
   SPI_Init(SPI1, &SPI_InitStructure);									 //wpisanie parametrow do struktury SPI1
   SPI_SSOutputCmd(SPI1, ENABLE);
   SPI_Cmd(SPI1, ENABLE);												 //funkcja zalaczajaca interfejs szerego SPI
}


void RCC_Config(void)
{
	  RCC_DeInit();                                          //Reset ustawien RCC
	  RCC_HSEConfig(RCC_HSE_ON);                             //Wlaczenie HSE



		RCC_HCLKConfig(RCC_SYSCLK_Div1);                     //ustaw HCLK=SYSCLK
		RCC_PCLK2Config(RCC_HCLK_Div1);                      //ustaw PCLK2=HCLK
		RCC_PCLK1Config(RCC_HCLK_Div1);                      //ustaw PCLK1=HCLK/2
	    RCC_PLLConfig(RCC_PLLSource_PREDIV1, RCC_PLLMul_9); //ustaw PLLCLK = HSE*9 czyli 8MHz * 9 = 72 MHz

	    RCC_PLLCmd(ENABLE);                                  //wlacz PLL
	    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET){}  //odczekaj na poprawne uruchomienie PLL
	    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);           //ustaw PLL jako zrodlo sygnalu zegarowego
	    while(RCC_GetSYSCLKSource() != 0x08){}                //odczekaj az PLL bedzie sygnalem zegarowym systemu

	  /*kod zwiazany z konfiguracja sygnalow zegarowych potrzebnych w programie peryferiow*/

	        RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE); //wlaczenie wyjsc alternatywnych


			RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
			RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);
			RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);


			RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE); //zalaczenie taktowania SPI1 dla wyswietlacza
			RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2, ENABLE);  //zalaczenie taktowania SPI2
			RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE); //wlacz taktowanie TIM3


			//Zalaczenie szyn odpowiedzialnych za pomiary wartosci
			RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE); //wlacz taktowanie ADC1
			//Podzia czestotliwosci dla ADC
			RCC_ADCCLKConfig(RCC_PCLK2_Div8);

			RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);   //wlacz taktowanie DMA

			RCC_APB1PeriphClockCmd(RCC_APB1Periph_DAC, ENABLE);  //zalaczenie przetwornika DAC

	        SystemCoreClockUpdate ();   //aktualizacja zmiennych potrzebnych w bibliotekach do przeliczenia wspolczynnikow
}

//Konifurowanie PWM dla mozliwosci zmiany jasnosci
void TIM_Config(void)
{
  //konfigurowanie licznikow

	//***************************************************************************************************
	//Poczatek konfiguracja timer3 odpowiedzialnego za jasnosc wyswietlacza
	//***************************************************************************************************

  TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
  TIM_OCInitTypeDef  TIM_OCInitStructure;

  // Konfiguracja TIM3
  // Ustawienia ukladu podstawy czasu
  TIM_TimeBaseStructure.TIM_Prescaler = 0;         //bez prescelera (prescaler=1)
  TIM_TimeBaseStructure.TIM_Period = 7999ul;      //czestotliwosc PWM =5,86kHz  poniewaz 72Mhz/8000=9kHz
  TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

  TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);


  // Konfiguracja kanalu 3 - uzywamy kanalu 3 poniewaz jego wyjscie jest na GPIOB0
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1; //tryb pwm
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = 0ul;          //poczatkowe wypelnienie maksymalne (TIMpulse/Timperiod)*100%=100%
  TIM_OCInitStructure.TIM_OCPolarity=TIM_OCPolarity_High;
  TIM_OC3Init(TIM3, &TIM_OCInitStructure);

  TIM_OC3PreloadConfig(TIM3, TIM_OCPreload_Enable);//wlaczenie buforowania

  TIM_ARRPreloadConfig(TIM3, ENABLE);//wlaczenie buforowania
  // Wlaczenie timera
  TIM_Cmd(TIM3, ENABLE);

}


//Funkcja konfigurujaca wewnetrznego watchdoga
#if enablewatchdog==1
void IWDT_config(void)
{
	IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);  //zezwolenie na zapis do rejestrow IWDG

	IWDG_SetPrescaler(IWDG_Prescaler_256); // ustawienie preskalera IWDT na 256 czyli 40kHz/256=156Hz
	IWDG_SetReload(0x270);      //ustawienie wartosci przeladowania na okolo 3s
	IWDG_ReloadCounter();       //przeladowanie IWDT
	IWDG_Enable();              //zalaczenie iwdt
}
#endif

//Konfiguracja ukladu DMA dla ADC
void DMA_Config(void)
{
  //konfigurowanie DMA

  #define ADC1_DR_Address 0x4001244C;                                             //adres rejestru ADC1->DR

  DMA_InitTypeDef DMA_InitStructure;

  DMA_DeInit(DMA1_Channel1);                                                      //Usun ewentualna poprzednia konfiguracje DMA

  DMA_InitStructure.DMA_PeripheralBaseAddr = (unsigned long int)ADC1_DR_Address;  //Adres docelowy transferu
  DMA_InitStructure.DMA_MemoryBaseAddr = (unsigned long int)valueADC;            //Adres poczatku bloku do przeslania
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;                              //Kierunek transferu
  DMA_InitStructure.DMA_BufferSize = 2;                                           //Liczba elementow do przeslania (dlugosc bufora)
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;                //Wylaczenie automatycznego zwiekszania adresu po stronie ADC
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;                         //Wylaczenie automatycznego zwiekszania adresu po stronie pamieci (bufora)
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;     //Rozmiar pojedynczych przesylanych danych po stronie ADC (HalfWord = 16bit)
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;             //Rozmiar pojedynczych przesylanych danych po stronie pamieci (bufora)
  DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;                                 //Tryb dzialania kontrolera DMA - powtarzanie cykliczne
  DMA_InitStructure.DMA_Priority = DMA_Priority_High;                             //Priorytet DMA - wysoki
  DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;                                    //Wylaczenie obslugi transferu z pamieci do pamieci
  DMA_Init(DMA1_Channel1, &DMA_InitStructure);                                    //Zapis konfiguracji

  //Wlacz DMA, kanal 1
  DMA_Cmd(DMA1_Channel1, ENABLE);
}

//Konfiguracja ADC
void ADC_Config(void)
{
  //konfigurowanie przetwornika AC
  ADC_InitTypeDef ADC_InitStructure;

	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;                  //Jeden przetwornik, praca niezalezna
	ADC_InitStructure.ADC_ScanConvMode = ENABLE;                        //Pomiar jednego kanalu, brak skanowania kanalow
    ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;                  //Pomiar w trybie ciaglym
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;	//Brak wyzwalania zewnetrznego
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;            	//Wyrownanie danych do prawej - 12 mlodszych bitow znaczacych
	ADC_InitStructure.ADC_NbrOfChannel = 2; 	                          //Liczba uzywanych kanalow =1
	ADC_Init(ADC1, &ADC_InitStructure);                                 //Incjalizacja przetwornika

    ADC_RegularChannelConfig(ADC1, ADC_Channel_1, 1, ADC_SampleTime_239Cycles5);
    ADC_RegularChannelConfig(ADC1, ADC_Channel_2, 2, ADC_SampleTime_239Cycles5);
                                                                      //Kanal 1 - GPIOA1
                                                                      //Grupa podstawowa, czas probkowania 41,5+12,5 cykli (calkowity czas przetwarania)
   ADC_DMACmd(ADC1,ENABLE);                                            //Wlaczenie DMA dla ADC
   ADC_Cmd(ADC1, ENABLE);	                                            //Wlacz ADC1

   ADC_ResetCalibration(ADC1);	                                        //Reset rejestrow kalibracyjnych ADC1
   while(ADC_GetResetCalibrationStatus(ADC1));	                        //Odczekanie na wykonanie resetu
   ADC_StartCalibration(ADC1);	                                        //Kalibracja ADC1
   while(ADC_GetCalibrationStatus(ADC1));    	                        //Odczekanie na zakonczenie kalibracji ADC1
}

void DAC_Config(void)
{

	DAC_InitTypeDef   DAC_InitStructure;

	DAC_DeInit();

  DAC_InitStructure.DAC_Trigger = DAC_Trigger_None;										//Wyzwalanie ciagle
  DAC_InitStructure.DAC_WaveGeneration = DAC_WaveGeneration_None;			//Wylaczenie generacji fsli
  DAC_InitStructure.DAC_OutputBuffer = DAC_OutputBuffer_Enable;				//Zalaczenie buffora wyjsciowego

  DAC_Init(DAC_Channel_1, &DAC_InitStructure);							   //inicjalizacja
  DAC_Init(DAC_Channel_2, &DAC_InitStructure);

	DAC_Cmd(DAC_Channel_1, ENABLE);											//zalaczenie
	DAC_Cmd(DAC_Channel_2, ENABLE);

}

//--------------------------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------------------------------------------------------------
//---------------------------------------- FUNKCJE UYTKOWE ---------------------------------------------------------------------------------------------

//funkcja zwracajaca wartosci wspolrzednych po dotknieciu ekranu
uint8_t gettouchvalue(void)
{
	float temp1, temp2;
	uint8_t k;
	uint8_t temp=0;

	uint32_t bigx=0, bigy=0;

	if(!touchms)
	{
		touchms=10; //5ms pomiedzy pomiarami


		buftx[globalk]=ADS_Read_AD(CMD_RDY);
		bufty[globalk]=ADS_Read_AD(CMD_RDX);


		//Wyelyminowanie dodatkowych zaklocen (fluktuacji )
		if(buftx[globalk]>100  && buftx[globalk]<4000  && bufty[globalk]>100  && bufty[globalk]<4000)
		globalk++;
		else globalk=0;
	}

 if(globalk==4)
{
			 			k=globalk;
			 			//wykasowanie poprzedniego wyniku
			 			globalk=0;

			 			for(k=0;k<4; k++)
			 			{

			 				bigx+=buftx[k];
			 				bigy+=bufty[k];
			 			}

			 			Xtouch=bigx/k;
			 			Ytouch=bigy/k;

		if(wykalibrowano)
		{
			if(Ytouch>xcal2 && Ytouch<xcal1 && Xtouch>ycal2 && Xtouch<ycal1) //jezeli odczytana wartosc miesci sie w granicach zmiennych granicznych
			{

			temp1 = aX*(float)Xtouch + bX*(float)Ytouch + dX;
			temp2 = aY*(float)Xtouch + bY*(float)Ytouch + dY;

			Xtouch=temp1;
			Ytouch=temp2;
			temp=1;
		    }
			else
				{
					Xtouch=0;
					Ytouch=0;
					temp=0;
					globalk2=0;  //wykasowanie zmiennej odpowiedzialnej za odczyt blednych dotkniec
				}
			return temp;
		}
     }
	 return temp;
}

void calibrationtouch(void)
{

	uint16_t touchval[3][3] = {{1,1,1}, {1,1,1}, {1,1,1}};   //Talbica wartosci z przetwornika przy czym
	//pierwszy element x, drugi y a trzeci 1 - poniewaz tak wyglada macierz
	int32_t transtouch[3][3];                                     //macierz trans dla macierzy touchval
	uint32_t dettouch; 										  //wyznacznik macierzy touchval
	//wspolrzende punktow ktore beda rysowane
	uint16_t wsppx[3]={15, 160, 305};
	uint16_t wsppy[3]={15, 225, 20};
	float invtouch[3][3];                       // inwersja  macierzy A
	float temporarytouch;

	uint8_t ip, jp;

	ILI9341_Clear(White); //Wyczyszczenie wywietlacza biaym tem
    chosefont=6;
	tft_mputs( 20, 90, "Prosz dotkn i trzyma", Red2, White, 0);
	tft_mputs( 20, 110,"   widoczny kwardat     ", Red2, White, 0);

#if enablewatchdog==1
	clearwdt;           //Wykasowanie watchdog
#endif
	TIMERUS=20000;  //odczekanie 2s
	while(TIMERUS);

												  //Pierwszy punkt
	full_rect( 14, 14, 16, 16, Red1);            //punkt x=15 y=15

#if enablewatchdog==1
	clearwdt;           //Wykasowanie watchdog
#endif

	TIMERUS=20000;  //odczekanie 2s
	while(TIMERUS)
	{
		gettouchvalue();   //pobieraie wartosci z kontrolera oraz usrednianie
	}


	touchval[0][0] = Xtouch;
	touchval[0][1] = Ytouch;

	full_rect( 14, 14, 16, 16, White);			  //Drugi punkt
	full_rect( 159, 224, 161, 226, Red1);        //punkt x=160, y=225

#if enablewatchdog==1
	clearwdt;           //Wykasowanie watchdog
#endif

	TIMERUS=20000; //odczekanie 2s
	while(TIMERUS)
	{
		gettouchvalue();   //pobieraie wartosci z kontrolera oraz usrednianie
	}


	touchval[1][0] = Xtouch;
	touchval[1][1] = Ytouch;

	full_rect( 159, 224, 161, 226, White);        //Trzeci punkt
	full_rect( 304, 19, 306, 21, Red1);        // punkt x=305 y=120

#if enablewatchdog==1
	clearwdt;           //Wykasowanie watchdog
#endif

	TIMERUS=20000; //odczekanie 2s
	while(TIMERUS)
	{
		gettouchvalue();   //pobieraie wartosci z kontrolera oraz usrednianie
	}

#if enablewatchdog==1
	clearwdt;           //Wykasowanie watchdog
#endif

	touchval[2][0] = Xtouch;
	touchval[2][1] = Ytouch;

	ILI9341_Clear(White);   //wyczyszczenie ekranu

	//obliczanie wyznacznika macierzy pobranych polozen z rownania
	dettouch = ((touchval[0][0])*(((touchval[1][1])*(touchval[2][2]))-((touchval[1][2])*(touchval[2][1])))) +
	((touchval[0][1])*(((touchval[1][2])*(touchval[2][0])) - ((touchval[2][2])*(touchval[1][0])))) +
	((touchval[0][2])*(((touchval[1][0])*(touchval[2][1])) - ((touchval[1][1])*(touchval[2][0]))));

	//obliczanie elementow macierzy odwroconej za pomoca wzorow - na minior (elementy jeszcze nie podzielone przez wyznacznik)
	transtouch[0][0] = (touchval[1][1] * touchval[2][2]) - (touchval[1][2] * touchval[2][1]);
	transtouch[1][0] = (touchval[1][2] * touchval[2][0]) - (touchval[1][0] * touchval[2][2]);
	transtouch[2][0] = (touchval[1][0] * touchval[2][1]) - (touchval[1][1] * touchval[2][0]);
	transtouch[0][1] = (touchval[2][1] * touchval[0][2]) - (touchval[0][1] * touchval[2][2]);
	transtouch[1][1] = (touchval[0][0] * touchval[2][2]) - (touchval[2][0] * touchval[0][2]);
	transtouch[2][1] = (touchval[2][0] * touchval[0][1]) - (touchval[0][0] * touchval[2][1]);
	transtouch[0][2] = (touchval[0][1] * touchval[1][2]) - (touchval[1][1] * touchval[0][2]);
	transtouch[1][2] = (touchval[0][2] * touchval[1][0]) - (touchval[0][0] * touchval[1][2]);
	transtouch[2][2] = (touchval[0][0] * touchval[1][1]) - (touchval[0][1] * touchval[1][0]);

	//Obliczenie odwrotnosci
	temporarytouch = 1/((float)dettouch);

	//obliczenie wszystkich elementow odwrotnej macierzy
	for(ip=0;ip<3;ip++)
	{
	for(jp=0;jp<3;jp++)
	{
	invtouch[ip][jp] = temporarytouch * ((float)transtouch[ip][jp]);
	}
	}

	aX = invtouch[0][0]*(float)wsppx[0] + invtouch[0][1]*(float)wsppx[1] +
	invtouch[0][2]*(float)wsppx[2];
	bX = invtouch[1][0]*(float)wsppx[0] + invtouch[1][1]*(float)wsppx[1] +
	invtouch[1][2]*(float)wsppx[2];
	dX = invtouch[2][0]*(float)wsppx[0] + invtouch[2][1]*(float)wsppx[1] +
	invtouch[2][2]*(float)wsppx[2];

	aY = invtouch[0][0]*(float)wsppy[0] + invtouch[0][1]*(float)wsppy[1] +
		 invtouch[0][2]*(float)wsppy[2];
	bY = invtouch[1][0]*(float)wsppy[0] + invtouch[1][1]*(float)wsppy[1] +
		 invtouch[1][2]*(float)wsppy[2];
	dY = invtouch[2][0]*(float)wsppy[0] + invtouch[2][1]*(float)wsppy[1] +
		 invtouch[2][2]*(float)wsppy[2];

	//w tym miejscu nastepuje zapis wspolczynnikow kalibracji do pamieci flash
	//-------------------------------------------------------------------
	Flash_zapis2();
	//-------------------------------------------------------------------

}

//---------------------------------------------------------------------------------------------------------------
//---------------------- Funkcje obslugujace pamiec flash uC-----------------------------------------------------

//Funkcja odczytujaca wspolczynniki dla kalibracji touchpanelu
void Flash_odczyt2(void)
{
	uint32_t *p;
	uint32_t  t;
	unsigned int wspolczynnikkalibracji[6];

	 union
	    {
	     unsigned int i;
	     float f;
	     }var1,var2, var3, var4, var5, var6;


	p=(uint32_t *)(BaseAdresFlash+OffsetNazwy2+4);

	for(t=0; t<=5; t++)
	{
	 wspolczynnikkalibracji[t]=*p; //przepisanie wartosci z fla
	  p++;
	}

	var1.i = wspolczynnikkalibracji[0];
	var2.i = wspolczynnikkalibracji[1];
	var3.i = wspolczynnikkalibracji[2];
	var4.i = wspolczynnikkalibracji[3];
	var5.i = wspolczynnikkalibracji[4];
	var6.i = wspolczynnikkalibracji[5];

	aX=var1.f;
	bX=var2.f;
	dX=var3.f;
	aY=var4.f;
	bY=var5.f;
	dY=var6.f;

}

// Wersje drugie dla wspolczynnikow kalibracji panelu oraz kalibracji pomiaru
void Flash_zapis2(void)
{
	 uint32_t   i;
     volatile uint32_t  j;
     unsigned int wspolczynnikkalibracji[6];
     uint32_t *k;

     union
     {
     unsigned int i;
     float f;
     }var1,var2, var3, var4, var5, var6;

     //zrzutowanie zmiennych przecinkowych na unie aby mozna je bylo potem wpisac do pamieci flash jako unsigned int
     var1.f=aX;
     var2.f=bX;
     var3.f=dX;
     var4.f=aY;
     var5.f=bY;
     var6.f=dY;

     wspolczynnikkalibracji[0]=var1.i;
     wspolczynnikkalibracji[1]=var2.i;
     wspolczynnikkalibracji[2]=var3.i;
     wspolczynnikkalibracji[3]=var4.i;
     wspolczynnikkalibracji[4]=var5.i;
     wspolczynnikkalibracji[5]=var6.i;



 	k=(uint32_t *)(BaseAdresFlash + OffsetNazwy2);  //ustawienie wskaznika na adres komorki ktora jest flaga czy juz byl dokonany zapis i pierwsze uruchonmienie

 	if(*k!=0x256faba4  || flagaponownejkalibracji) //sprawdzenie po zalaczeniu czy pamiec juz byla uzywana i nie jest to pierwszy raz po zapisie
 	{

	//Odblokowanie flash

	FLASH_Unlock();

	NbrOfPage = (BANK1_WRITE_END_ADDR2 - BANK1_WRITE_START_ADDR2) / FLASH_PAGE_SIZE;

	FLASH_ClearFlag(FLASH_FLAG_EOP|FLASH_FLAG_PGERR|FLASH_FLAG_WRPRTERR);




  // Wykasowanie pamieci
  for(EraseCounter = 0; (EraseCounter < NbrOfPage) && (FLASHStatus == FLASH_COMPLETE); EraseCounter++)
  {
    FLASHStatus = FLASH_ErasePage(BANK1_WRITE_START_ADDR2 + (FLASH_PAGE_SIZE * EraseCounter));
  }


		while(FLASH_COMPLETE!=FLASH_ProgramWord(BaseAdresFlash + OffsetNazwy2, 0x256faba4))   //zmienna blokujaca nadpisanie przy uruchomieniu
			{
			}
		j=(BaseAdresFlash+OffsetNazwy2+4);


		//W tym miejscu nastepuje wpisanie zmiennych kalibracyjnych touchpanelu

		for(i=0; i<=5; i++)
		{
			 while(FLASH_COMPLETE!=FLASH_ProgramWord(j, wspolczynnikkalibracji[i]))
			 {
			 }
			 j=j+4;

	    }

	FLASH_Lock();  //zablokowanie dostepu do pamieci flash
 	}
}


//Funkcja odczytujaca wspolczynniki dla kalibracji touchpanelu
void Flash_odczytustawien(void)
{
	uint32_t *p;
	uint32_t  t;

	p=(uint32_t *)(BaseAdresFlash+OffsetNazwy+4);

	for(t=0; t<=2; t++)
	{
	 flashzapis2[t]=*p; //przepisanie wartosci z flash
	  p++;
	}

}

// Wersje drugie dla wspolczynnikow kalibracji panelu oraz kalibracji pomiaru
void Flash_zapisustawien(void)
{
	 uint32_t   i;
     volatile uint32_t  j;


	//Odblokowanie flash

	FLASH_Unlock();

	NbrOfPage = (BANK1_WRITE_END_ADDR - BANK1_WRITE_START_ADDR) / FLASH_PAGE_SIZE;

	FLASH_ClearFlag(FLASH_FLAG_EOP|FLASH_FLAG_PGERR|FLASH_FLAG_WRPRTERR);




  // Wykasowanie pamieci
  for(EraseCounter = 0; (EraseCounter < NbrOfPage) && (FLASHStatus == FLASH_COMPLETE); EraseCounter++)
  {
    FLASHStatus = FLASH_ErasePage(BANK1_WRITE_START_ADDR + (FLASH_PAGE_SIZE * EraseCounter));
  }


		while(FLASH_COMPLETE!=FLASH_ProgramWord(BaseAdresFlash + OffsetNazwy, 0x55aef09b))   //zmienna blokujaca nadpisanie przy uruchomieniu
			{
			}

		j=(BaseAdresFlash+OffsetNazwy+4);


		//W tym miejscu nastepuje wpisanie zmiennych ustawien
		for(i=0; i<=2; i++)
		{
			 while(FLASH_COMPLETE!=FLASH_ProgramWord(j, flashzapis2[i]))
			 {
			 }
			 j=j+4;
	    }

	FLASH_Lock();  //zablokowanie dostepu do pamieci flash

}

//-------------------------------------------------------------------------------------------------------------
//Funkcja odpowiedzialna za stan podswietlenia wywietlacza

void dimingled(uint8_t diming)
{
		TIM3->CCR3=1000+(diming*79);
}


//--------------------------------------------------------------------------------------------------------------
//Funkcja testowa panelu dotykowego
void testingtouch(void)
{
	 chosefont=6;
	 tft_mputs( 20, 10, "  Funkcja testowa   ", Red2, White, 0);
	 tft_mputs( 20, 30, "  panelu dotykowego ", Red2, White, 0);
	 tft_mputs( 20, 60, "  wsp. X ", Red2, White, 0);
	 tft_mputs( 20, 130,"  wsp. Y ", Red2, White, 0);
	 while(1)
	 {

		 chosefont=3; //wybranie czcionnki
		 lcdputvalue(40, 80, Xtouch, Blue1);
		 lcdputvalue(40, 150, Ytouch, Blue1);

		 while(1)
		 {
			 if( gettouchvalue())
		 	 {
			 lcdputvalue(40, 80, Xtouch, Blue1);
		     lcdputvalue(40, 150, Ytouch, Blue1);
		 	 }
		 }

#if enablewatchdog==1
	clearwdt;           //Wykasowanie watchdog
#endif
	 }
}

//---------------------------------------------------------------------------------------------------------------------------------
//Funkcja wyswietlajaca na ekranie aktualna wartosc pradu oraz napiecia
void putvaluecurrentvoltage(uint8_t *firstdraw, uint16_t *current, uint16_t *voltage)
{
	static uint16_t tempcurrent=0, tempvoltage=0;
	static uint8_t  przecinekV=1, przecinekA=1;

	if(*firstdraw)
	{
		*firstdraw=0;
		//tft_round_rect(10, 50, 300, 140, 20, Blue4, Blue4);  //Funkcja rysujaca zaokraglony prostokat
		putpictures2(10, 50, 300, 140, tlo);

		tempcurrent=0xffff;
		tempvoltage=0xffff;
		przecinekA=1;    //zezwolenie na jednorazowe narysowanie przecinka
		przecinekV=1;
	}

	if(tempvoltage!=*voltage)
	{
		tempvoltage=*voltage;
		valueV(57, 75, voltage, przecinekV);
		przecinekV=0;
	}

	if(tempcurrent!=*current)
	{
		tempcurrent=*current;
		valueA(57, 130, current, przecinekA);
		przecinekA=0;
	}
}
//---------------------------------------------------------------------------------------------------------
//Funkcja sluzaca do wyswietlenia aktualnej wartosci napiecia
void valueV(uint16_t x, uint16_t y, uint16_t *measure, uint8_t przecinek)
{
	char tempmeasure[3];
	char tempmeasure2[3];
	uint16_t temptemp[2];

	tempmeasure[0]=*measure/1000;
	temptemp[0]=*measure - tempmeasure[0]*1000;
	tempmeasure[1]=temptemp[0]/100;
	temptemp[1]=temptemp[0] - tempmeasure[1]*100;
	tempmeasure2[0]=temptemp[1]/10;
	tempmeasure2[1]=*measure%10;


	tempmeasure2[2]=0x00;
	tempmeasure[2]=0x00;


	tempmeasure[0]+=0x30;
	tempmeasure[1]+=0x30;

	tempmeasure2[0]+=0x30;
	tempmeasure2[1]+=0x30;

	chosefont=5;
	if(przecinek)
	{
		tft_mputs( x+58, y, ",", White, Blue4, 0);
		tft_mputs( x+170, y, "V", White, Blue4, 0);
	}

	tft_mputs( x, y, tempmeasure, White, Blue4, 0);
	tft_mputs( x+83, y, tempmeasure2, White, Blue4, 0);

}

//Funkcja wyswietlajaca aktualna wartosc pradu
void valueA(uint16_t x, uint16_t y, uint16_t *measure, uint8_t przecinek)
{
	char tempmeasure[2];
	char tempmeasure2[4];
	uint16_t temptemp[2];

	tempmeasure[0]=*measure/1000;
	temptemp[0]=*measure - tempmeasure[0]*1000;
	tempmeasure2[0]=temptemp[0]/100;
	temptemp[1]=temptemp[0] - tempmeasure2[0]*100;
	tempmeasure2[1]=temptemp[1]/10;
	tempmeasure2[2]=*measure%10;


	tempmeasure2[3]=0x00;
	tempmeasure[1]=0x00;

	tempmeasure[0]+=0x30;
	tempmeasure2[0]+=0x30;
	tempmeasure2[1]+=0x30;
	tempmeasure2[2]+=0x30;

	chosefont=5;
	if(przecinek)
	{
		tft_mputs( x+25, y, ",", White, Blue4, 0);
		tft_mputs( x+170, y, "A", White, Blue4, 0);
	}

	tft_mputs( x, y, tempmeasure, White, Blue4, 0);
	tft_mputs( x+50, y, tempmeasure2, White, Blue4, 0);

}
//Funkcja animujaca znak dotyku lub znak zamkniecia podmenu
void animationtouch(uint16_t x, uint16_t y, uint8_t touchclose)
{
	static uint8_t number=0, flaga=1;
	const uint16_t colors[16]={0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x08A4, 0x1149, 0x19CC, 0x2271, 0x2B36, 0x3BFA,  0x64DB, 0x9DDD, 0xCEFE, 0xFFFF, 0xFFFF};

	if(!animetimer1)
	{
		if(flaga) number++; else number--;
		if(number>14) flaga=0; else if(number==0) flaga=1;

		animetimer1=800;
	}

	if(touchclose)
	{
		putmonograph(x+5, y, colors[number], White, 32, 32,touchicon);
	}
	else
	{
		putmonograph(x, y, colors[number], White, 32, 32,closeicon);
	}
}
//Funkcja obslugujaca panel dotykowy dla podmenu 0 - menu glownego
void touchmenu0(void)
{
	if(Xtouch>270 && Xtouch<320 && Ytouch>190 && Ytouch<240)
				 	 {
				 		menu=1;
				 		full_rect3( 55, 70, 290, 185, Blue4); //Wyczyszczenie wyswietlacza z aktualnie mierzonych wartosci
				 		varshow=1;
				 		varfirstdraw2=1;  //zezwolenie na narysowanie strzalek po wejscu do menu w ktorym dokonujemy ustawien pradu oraz napiecia
				 		varupdown=0;
				 		varfirstdraw3=1;
				 		varselect=1;
				 		full_rect3( 279, 198, 320, 240, White); //Wyczyszczenie ikonki bialym tlem
				 		tempsetvoltage=flashzapis2[0];          //przepisanie tymczasowych ustawien pradu i napiecia
				 		tempsetcurrent=flashzapis2[1];
				 		ttvoltage=flashzapis2[0];
				 		ttcurrent=flashzapis2[1]; //ustawienie aktualnie ustawionego napiecia i pradu jezeli nastapi zmiana to wtedy nastapi zapis do pamieci flash nowych wartosci

				 	 }
}
//obsluga panelu dotykowego w podmenu zmiany ustawienia
void touchmenu1(void)
{
	if(Xtouch>270 && Xtouch<320 && Ytouch>200 && Ytouch<240)
					 	 {
					 		menu=0;
					 		varfirstdraw1=1;
					 		full_rect3( 279, 198, 320, 240, White); //Wyczyszczenie ikonki bialym tlem

					 		//Zapis do pamieci flash oraz zmiana wartosci DAC
					 		if((ttvoltage!=flashzapis2[0]) || (ttcurrent!=flashzapis2[1])) //jezeli nastapila zmiana wartosci to nalezy zapisac nowe wartosci do pamieci flash oraz zmiana wartosci DAC
					 		{
					 			Flash_zapisustawien();
					 			setdacs(flashzapis2[1], flashzapis2[0]);
					 			varrange=1;          //zezwolenie na zmiane zakresu po zmianie napiecia
					 			varfirstregulation=1; //jezeli nastapila zmiana wartosci to zaczynaj regulacje od poczatku
					 		}

					 		timervalue=0; //wykasowanie timera czyli zezwolenia na pierwsze narysowanie wartosci
					 	 }
	//obsluga zaznaczenia
	if(Xtouch>47 && Xtouch<90 && Ytouch>65 && Ytouch<120)
						 	 {
								if(varselect==1)
									{
									varselect=0;
									checkvalue();
									}
								else varselect=1;
						 	 }
	if(Xtouch>91 && Xtouch<130 && Ytouch>65 && Ytouch<120)
							 {
								if(varselect==2)
								{
									varselect=0;
									checkvalue();
								}
								else varselect=2;
							 }
	if(Xtouch>131 && Xtouch<163 && Ytouch>65 && Ytouch<120)
							{
							    if(varselect==3)
							    {
							    	varselect=0;
							    	checkvalue();
							    }
							    	else varselect=3;
						    }
	if(Xtouch>164 && Xtouch<197 && Ytouch>65 && Ytouch<120)
							{
		                        if(varselect==4)
		                        	{
		                        	varselect=0;
		                        	checkvalue();
		                        	}
		                        else varselect=4;
							}
	if(Xtouch>57 && Xtouch<98 && Ytouch>125 && Ytouch<185)
							{
								if(varselect==5)
								{
									varselect=0;
									checkvalue();
								}
								else varselect=5;
							}
	if(Xtouch>99 && Xtouch<112 && Ytouch>125 && Ytouch<185)
							{
								if(varselect==6)
									{
									varselect=0;
									checkvalue();
									}
								else varselect=6;
							}
	if(Xtouch>113 && Xtouch<145 && Ytouch>125 && Ytouch<185)
							{
								if(varselect==7)
								{
									varselect=0;
									checkvalue();
								}
								else varselect=7;
							}
	if(Xtouch>146 && Xtouch<197 && Ytouch>125 && Ytouch<185)
							{
								if(varselect==8)
								{
									varselect=0;
									checkvalue();
								}
								else varselect=8;
							}
	//Inkrementacja lub dekrementacja wartosci pradu lub napiecia
	if(Xtouch>240 && Xtouch<320 && Ytouch>65 && Ytouch<120)
							{
		                       if(varselect)
		                       {
		                    	   varchangevalue=1;  //zezwolenie na zmiane wyswietlenia

		                    	   switch(varselect)
		                    	   {
		                    	   case 1:
		                    	   {
		                    		   numbers[0]++;
		                    		   if(numbers[0]>9) numbers[0]=0;
		                    		   break;
		                    	   }
		                    	   case 2:
		                    	   {
		                    		   numbers[1]++;
		                    		   if(numbers[1]>9) numbers[1]=0;
		                    		   break;
		                    	   }
		                    	   case 3:
		                    	   {
		                    		   numbers[2]++;
		                    		   if(numbers[2]>9) numbers[2]=0;
		                    		   break;
		                    	   }
		                    	   case 4:
		                    	   {
		                    		   numbers[3]++;
		                    		   if(numbers[3]>9) numbers[3]=0;
		                    		   break;
		                    	   }
		                    	   case 5:
		                    	   {
		                    		   numbers[4]++;
		                    		  if(numbers[4]>9) numbers[4]=0;
		                    		   break;
		                    	   }
		                    	   case 6:
		                    	   {
		                    		   numbers[5]++;
		                    		   if(numbers[5]>9) numbers[5]=0;
		                    		   break;
		                    	   }
		                    	   case 7:
		                    	   {
		                    		   numbers[6]++;
		                    		  if(numbers[6]>9) numbers[6]=0;
		                    		   break;
		                    	   }
		                    	   case 8:
		                    	   {
		                    		   numbers[7]++;
		                    		   if(numbers[7]>9) numbers[7]=0;
		                    		   break;
		                    	   }
		                    	   }
		                    	   tempsetvalue();
		                       }
							}
	if(Xtouch>240 && Xtouch<320 && Ytouch>125 && Ytouch<185)
							{
		                       if(varselect)
				               {
		                    	   varchangevalue=1;  //zezwolenie na zmiane wyswietlenia wybranej wartosci

		                    	  	 switch(varselect)
		                    	  	{
		                    	  		 case 1:
		                    	  		  {
		                    	  		      numbers[0]--;
		                    	  		      if(numbers[0]>250) numbers[0]=9;
		                    	  		      break;
		                    	  		  }
		                    	  		  case 2:
		                    	  		    {
		                    	  		       numbers[1]--;
		                    	  		       if(numbers[1]>250) numbers[1]=9;
		                    	  		       break;
		                    	  		    }
		                    	  		  case 3:
		                    	  		    {
		                    	  		       numbers[2]--;
		                    	  		       if(numbers[2]>250) numbers[2]=9;
		                    	  		       break;
		                    	  		    }
		                    	  		  case 4:
		                    	  		    {
		                    	  		       numbers[3]--;
		                    	  		       if(numbers[3]>250) numbers[3]=9;
		                    	  		       break;
		                    	  		    }
		                    	  		  case 5:
		                    	  		    {
		                    	  		        numbers[4]--;
		                    	  		        if(numbers[4]>250) numbers[4]=9;
		                    	  		        break;
		                    	  		    }
		                    	  		  case 6:
		                    	  		    {
		                    	  		         numbers[5]--;
		                    	  		         if(numbers[5]>250) numbers[5]=9;
		                    	  		         break;
		                    	  		    }
		                    	  		   case 7:
		                    	  		    {
		                    	  		         numbers[6]--;
		                    	  		         if(numbers[6]>250) numbers[6]=9;
		                    	  		         break;
		                    	  		    }
		                    	  		    case 8:
		                    	  		    {
		                    	  		         numbers[7]--;
		                    	  		         if(numbers[7]>250) numbers[7]=9;
		                    	  		         break;
		                    	  		    }
		                    	  		}
		                    	 	 tempsetvalue();
		                    	   }

							}
}


//wyswietlenie lub schowanie aniowanych strzalek inkrementacji lub  dekrementacji danej cyfry
void putincdec(uint16_t x, uint16_t y, uint8_t *show, uint8_t* firstdraw2)
{
	static uint8_t tempshow=0;
	static uint8_t number=0;
	uint8_t flagadif=0;
	static uint8_t flagapomoc=0;

	if(*firstdraw2)
	{
		*firstdraw2=0;
	  	number=0;
	  	tempshow=0xff;
	  	flagapomoc=0x00;
	}

	if((tempshow!=*show) || flagapomoc)
	{
		flagapomoc=(*show)+1;
		if(!animetimer2)
		{

		if(flagapomoc==2)
			{
			number++;
			if(number>8)
				{
				number=8;
				flagadif=1;
				flagapomoc=0;
				}
			}
		else if(flagapomoc==1)
		{
			if(number) number--;
			if(!number)
				{
				flagadif=1;
				flagapomoc=0;
				}
		}
		putmonograph(x, y, changecolors[number], Blue4, 32, 32,changeup);
		putmonograph(x, y+50, changecolors[number], Blue4, 32, 32,changedown);

		if(flagadif)  tempshow=*show; else animetimer2=1000;

		}
	}
}

//narysowanie aktualnie wyswietlonych parametrow - pradu oraz napiecia
void putactualvalue(uint16_t x, uint16_t y, uint8_t *firstdraw3, uint8_t *zaznaczenie, uint8_t *show, uint8_t *changevalue)
{

	uint16_t temp1, temp2;
	static uint8_t tempselect=0;

	char cyfra1[2];
	char cyfra2[2];
	char cyfra3[2];
    char cyfra4[2];
    char cyfra5[2];
	char cyfra6[2];
	char cyfra7[2];
    char cyfra8[2];

	chosefont=5;

	if(*firstdraw3)
	{
		*firstdraw3=0;

		tft_mputs( x+58, y, ",", White, Blue4, 0);
		tft_mputs( x+170, y, "V", White, Blue4, 0);
		tft_mputs( x+25, y+55, ",", White, Blue4, 0);
		tft_mputs( x+170, y+55, "A", White, Blue4, 0);

		tempselect=0;

	}

	if((tempselect!=*zaznaczenie) || *changevalue)
	{
		if(*changevalue) *changevalue=0;

		cyfra1[0]=tempsetvoltage/1000;
		temp1=tempsetvoltage-cyfra1[0]*1000;
		cyfra2[0]=temp1/100;
		temp2=temp1-cyfra2[0]*100;
		cyfra3[0]=temp2/10;
		cyfra4[0]=tempsetvoltage%10;

		cyfra5[0]=tempsetcurrent/1000;
		temp1=tempsetcurrent-cyfra5[0]*1000;
		cyfra6[0]=temp1/100;
		temp2=temp1-cyfra6[0]*100;
		cyfra7[0]=temp2/10;
		cyfra8[0]=tempsetcurrent%10;

		numbers[0]=cyfra1[0];
		numbers[1]=cyfra2[0];
		numbers[2]=cyfra3[0];
		numbers[3]=cyfra4[0];
		numbers[4]=cyfra5[0];
		numbers[5]=cyfra6[0];
		numbers[6]=cyfra7[0];
		numbers[7]=cyfra8[0];

		cyfra1[0]+=0x30;
		cyfra2[0]+=0x30;
		cyfra3[0]+=0x30;
		cyfra4[0]+=0x30;
		cyfra5[0]+=0x30;
		cyfra6[0]+=0x30;
		cyfra7[0]+=0x30;
		cyfra8[0]+=0x30;

		cyfra1[1]=0x0;
		cyfra2[1]=0x0;
		cyfra3[1]=0x0;
		cyfra4[1]=0x0;
		cyfra5[1]=0x0;
		cyfra6[1]=0x0;
		cyfra7[1]=0x0;
		cyfra8[1]=0x0;

			if(*zaznaczenie==1) tft_mputs( x, y, cyfra1, Blue4, White, 0); else tft_mputs( x, y, cyfra1, White, Blue4, 0);
			if(*zaznaczenie==2) tft_mputs( x+33, y, cyfra2, Blue4, White, 0); else tft_mputs( x+33, y, cyfra2, White, Blue4, 0);
			if(*zaznaczenie==3) tft_mputs( x+83, y, cyfra3, Blue4, White, 0); else tft_mputs( x+83, y, cyfra3, White, Blue4, 0);
			if(*zaznaczenie==4) tft_mputs( x+116, y, cyfra4, Blue4, White, 0); else tft_mputs( x+116, y, cyfra4, White, Blue4, 0);
			if(*zaznaczenie==5) tft_mputs( x, y+55, cyfra5, Blue4, White, 0); else tft_mputs( x, y+55, cyfra5, White, Blue4, 0);
			if(*zaznaczenie==6) tft_mputs( x+50, y+55, cyfra6, Blue4, White, 0); else tft_mputs( x+50, y+55, cyfra6, White, Blue4, 0);
			if(*zaznaczenie==7) tft_mputs( x+88, y+55, cyfra7, Blue4, White, 0); else tft_mputs( x+88, y+55, cyfra7, White, Blue4, 0);
			if(*zaznaczenie==8) tft_mputs( x+121, y+55, cyfra8, Blue4, White, 0); else tft_mputs( x+121, y+55, cyfra8, White, Blue4, 0);

			if(*zaznaczenie) *show=1; else *show=0;

			tempselect=*zaznaczenie;

	}
}
//funkcja sprawdzajaca wartosci ustawione i przepisujace je do flashzapis jezeli zostaly prawidlowo ustawione
void checkvalue(void)
{
	tempsetvalue();

	if(tempsetvoltage>3000) flashzapis2[0]=3000; else flashzapis2[0]=tempsetvoltage;
	if(tempsetcurrent>2000) flashzapis2[1]=2000; else flashzapis2[1]=tempsetcurrent;

	tempsetvoltage=flashzapis2[0];
	tempsetcurrent=flashzapis2[1];
}
//funkcja obliczajaca tymczasowa wartosc pradu oraz napiecia
void tempsetvalue(void)
{
	tempsetvoltage=((numbers[0])*1000);
	tempsetvoltage=tempsetvoltage+((numbers[1])*100);
	tempsetvoltage=tempsetvoltage+((numbers[2])*10);
	tempsetvoltage=tempsetvoltage+numbers[3];
	tempsetcurrent=((numbers[4])*1000);
	tempsetcurrent=tempsetcurrent+((numbers[5])*100);
	tempsetcurrent=tempsetcurrent+((numbers[6])*10);
	tempsetcurrent=tempsetcurrent+ numbers[7];
}
//funkcja obliczajaca aktualna wartosc napiecia oraz pradu
void multimeter(uint16_t *valcur, uint16_t *valvolt)
{
	static uint8_t t1=0;
	static uint16_t bufforV[100];
	static uint16_t bufforA[100];
	uint32_t  temp1, temp2;

	bufforV[t1]=valueADC[0];
	bufforA[t1]=valueADC[1];

	t1++;
	if(t1>99) t1=0;

	temp1=0;
	temp2=0;
	for(uint8_t k=0; k<=99; k++)
	{
		temp1+=bufforV[k];
		temp2+=bufforA[k];
	}
	temp1=temp1/100;
	temp1=temp1*mfvoltage;
	*valvolt=temp1/4096;

	temp2=temp2/100;
	temp2=temp2*mfcurrent;
	*valcur=temp2/4096;

}
//funkcja ustawiajaca przetworniki ADC w zaleznosci od ustawionych wartosci pradow oraz napiecia
void setdacs(uint16_t currentdac, uint16_t voltagedac)
{
	uint32_t temp1, temp2;

	temp1=voltagedac*(0xfff);
	temp1=temp1/dacfvoltage;

	temp2=currentdac*(0xfff);
	temp2=temp2/dacfcurrent;

	DAC_SetChannel1Data(DAC_Align_12b_R, (uint16_t)temp1);
	DAC_SetChannel2Data(DAC_Align_12b_R, (uint16_t)temp2);
}
//Funkcja kontrolujaca diode LED oraz BUZZER
void controlLB(uint16_t *current, uint8_t *reg)
{
	static uint8_t flagabuz=1, flagaled=1;

	if(*current>1900) //jezeli wykryto prac zblizajacy sie do 2A to nastepuje uruchomienie Beeper
	{
	if(!timerbuzz)
		{
		timerbuzz=5000;

			if(flagabuz)
			{
				flagabuz=0;
				offBUZZER;
			}
			else
			{
				flagabuz=1;
				onBUZZER;
			}
		}
	}
	else
	{
		flagabuz=1;
		offBUZZER;  //wylaczenie buzzera
	}
	if(*current>1970)
	{
			onLED;
	}
	else
	{
		if(*reg)
		{
			if(!timerled)
			{
				if(flagaled)
				{
					timerled=300;
					onLED;
					flagaled=0;
				}
				else
				{
					flagaled=1;
					offLED;
					if(globalflag)
					{
						globalflag=0;
						timerled=5000;
					}
					else timerled=20000;

				}
			}
		}
		else
		{
			offLED;
		}
	}
}
//funkcja sluzaca do kontroli regulacji - zalaczenia lub wylaczenia
void controlregulation(uint8_t *regulation)
{
	if(regonoff)
	{
		*regulation=1;
	}
	else *regulation=0;
}
//Funkcja zmieniajaca zakres zasilacza
void changerange(uint8_t *changerange)
{
	//0, 500, 700, 1000, 1400, 1700, 2100, 2600, 3200 - zakresy napiec
	static uint8_t chrg=0;
    uint8_t i=0;

	if(*changerange)
	{
		if(!timerrange && (chrg==0))
		{
			timerrange=1000;
			offconverter;            //wylaczenie regulatora impulsowego
			chrg=1;
		}
		else if(!timerrange && (chrg==1))  //zmiana zakresu
		{
			//obliczenie zakresu
			for(i=0; i<=7; i++)
			{
				if((flashzapis2[0]>=ranges[i]) && (flashzapis2[0]<ranges[i+1])) break;
			}
			controlmux(i);

			timerrange=1000;
			chrg=2;
		}
		else if(!timerrange && (chrg==2))
		{

			onconverter;    //zalaczenie regulatora impulsowego
			*changerange=0; //wylaczenie zezwolenia na zmiane zakresu
			chrg=0;         //ustawienie zmiennej pomocniczej na sam poczatek
		}
	}
}
//Funcka dokonujaca doregulowywanie wartosci
void doregulation(uint8_t *regulation, uint8_t *changerange, uint16_t setvalue, uint16_t actualvalue, uint8_t *firstuse)
{
	signed long int temporary;
	static uint16_t incdecr=0, infinity=0, flaga2=0;
	uint32_t temp1;

	if(*firstuse)
	{
		*firstuse=0;
		infinity=flashzapis2[0];
	}

	if(*regulation && (!(*changerange)))  //jezeli jest wylaczona regulacja oraz nastapila juz zmiana zakresu to mozna dokonywac regulacji
	{
		flaga2=1;

		if(!timerregulation)
		{

				timerregulation=100; //reguacja co 10ms

				temporary=0;
				incdecr=0;

				temporary=setvalue-actualvalue;

				if(temporary & (0xfff000))  //jezeli wartosc jest ujemna
				{
					temporary=~temporary;
					temporary+=1;
					incdecr=1;
				}

				if(temporary>deltaregulation)
				{
					if(incdecr)  //zmniejszanie wartosci dac
					{
						if(infinity) infinity--;
						globalflag=1;
					}
					else        //zwiekszanie wartosci dac
					{
						if(infinity<3000) infinity++;
						globalflag=1;
					}

					temp1=(infinity)*(0xfff);
					temp1=temp1/dacfvoltage;

				    DAC_SetChannel1Data(DAC_Align_12b_R, (uint16_t)temp1);  //wpisanie nowej wartosci do przetwornika
				}
		}
	}
	else
	{
		if(flaga2)   //Po wylaczeniu regulacji jednorazowe ustawinie parametrow aktualnych
		{
		infinity=flashzapis2[0];
		incdecr=0;

		temp1=(flashzapis2[0])*(0xfff);
		temp1=temp1/dacfvoltage;
	    DAC_SetChannel1Data(DAC_Align_12b_R, (uint16_t)temp1);  //wpisanie nowej wartosci do przetwornika

		flaga2=0;
		}
	}
}
// Procedura obslugi multipleksera
void controlmux(uint8_t canal)
{
	uint8_t tempcanal=0;

	tempcanal=canal;
	tempcanal=~tempcanal;
	tempcanal=tempcanal&(0x07);

	if(tempcanal & 0x01) onAmux; else offAmux;
	if(tempcanal & 0x02) onBmux; else offBmux;
	if(tempcanal & 0x04) onCmux; else offCmux;
}
//Funkcja za pomoca ktorej mozemy ustawic podswietlenie
void setbacklight(uint8_t *firstdraw, uint8_t percent)
{
	uint8_t temppercent=0;

	if(*firstdraw)
	{
		*firstdraw=0;
		ILI9341_Clear(White); //Wyczyszczenie wywietlacza biaym tem

		chosefont=1;
		tft_mputs( 56, 34, "BACKLIGHT", Black1, White, 0);

		full_rect3( 5, 95, 315, 145, Black1);
		full_rect3( 10, 100, ((percent*3)+10), 140, Blue4);


		tft_round_rect(140, 180, 40, 20, 10, Blue4, Blue4);
		chosefont=6;
		tft_mputs( 147, 184, "OK", White, Blue4, 0);

		temppercent=percent;
	}

	if(temppercent!=percent)
	{
		temppercent=percent;


		full_rect3( 10, 100, ((percent*3)+10), 140, Blue4);
		full_rect3( (310-((100-percent)*3)), 100, 310, 140, Black1);
	}


}
